@@ -24,6 +24,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 // Qt
 #include <QApplication>
 #include <QDateTime>
+#include <QLineEdit>
 #include <QPushButton>
 #include <QShortcut>
 #include <QSplitter>
@@ -33,6 +34,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <QVBoxLayout>
 #include <QMenuBar>
 #include <QUrl>
+#include <QMouseEvent>
 #ifdef Q_OS_OSX
 #include <QFileOpenEvent>
 #endif
@@ -50,6 +52,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <KRecentFilesAction>
 #include <KStandardShortcut>
 #include <KToggleFullScreenAction>
+#include <KUrlComboBox>
 #include <KUrlNavigator>
 #include <KToolBar>
 #include <KXMLGUIFactory>
@@ -213,16 +216,16 @@ struct MainWindow::Private
     void setupContextManager()
     {
         mContextManager = new ContextManager(mDirModel, q);
-        connect(mContextManager, SIGNAL(selectionChanged()),
-            q, SLOT(slotSelectionChanged()));
-        connect(mContextManager, SIGNAL(currentDirUrlChanged(QUrl)),
-            q, SLOT(slotCurrentDirUrlChanged(QUrl)));
+        connect(mContextManager, &ContextManager::selectionChanged,
+            q, &MainWindow::slotSelectionChanged);
+        connect(mContextManager, &ContextManager::currentDirUrlChanged,
+            q, &MainWindow::slotCurrentDirUrlChanged);
     }
 
     void setupWidgets()
     {
         mFullScreenContent = new FullScreenContent(q, mGvCore);
-        connect(mContextManager, SIGNAL(currentUrlChanged(QUrl)), mFullScreenContent, SLOT(setCurrentUrl(QUrl)));
+        connect(mContextManager, &ContextManager::currentUrlChanged, mFullScreenContent, &FullScreenContent::setCurrentUrl);
 
         mCentralSplitter = new Splitter(Qt::Horizontal, q);
         q->setCentralWidget(mCentralSplitter);
@@ -234,18 +237,18 @@ struct MainWindow::Private
         mContentWidget = new QWidget(mCentralSplitter);
 
         mSaveBar = new SaveBar(mContentWidget, q->actionCollection());
-        connect(mContextManager, SIGNAL(currentUrlChanged(QUrl)), mSaveBar, SLOT(setCurrentUrl(QUrl)));
+        connect(mContextManager, &ContextManager::currentUrlChanged, mSaveBar, &SaveBar::setCurrentUrl);
         mViewStackedWidget = new QStackedWidget(mContentWidget);
         QVBoxLayout* layout = new QVBoxLayout(mContentWidget);
         layout->addWidget(mSaveBar);
         layout->addWidget(mViewStackedWidget);
-        layout->setMargin(0);
+        layout->setContentsMargins(0, 0, 0, 0);
         layout->setSpacing(0);
         ////
 
         mStartSlideShowWhenDirListerCompleted = false;
         mSlideShow = new SlideShow(q);
-        connect(mContextManager, SIGNAL(currentUrlChanged(QUrl)), mSlideShow, SLOT(setCurrentUrl(QUrl)));
+        connect(mContextManager, &ContextManager::currentUrlChanged, mSlideShow, &SlideShow::setCurrentUrl);
 
         setupThumbnailView(mViewStackedWidget);
         setupViewMainPage(mViewStackedWidget);
@@ -261,13 +264,13 @@ struct MainWindow::Private
 
         mThumbnailView->setFocus();
 
-        connect(mSaveBar, SIGNAL(requestSaveAll()),
-                mGvCore, SLOT(saveAll()));
-        connect(mSaveBar, SIGNAL(goToUrl(QUrl)),
-                q, SLOT(goToUrl(QUrl)));
+        connect(mSaveBar, &SaveBar::requestSaveAll,
+                mGvCore, &GvCore::saveAll);
+        connect(mSaveBar, &SaveBar::goToUrl,
+                q, &MainWindow::goToUrl);
 
-        connect(mSlideShow, SIGNAL(goToUrl(QUrl)),
-                q, SLOT(goToUrl(QUrl)));
+        connect(mSlideShow, &SlideShow::goToUrl,
+                q, &MainWindow::goToUrl);
     }
 
     void setupThumbnailView(QWidget* parent)
@@ -276,6 +279,7 @@ struct MainWindow::Private
         mBrowseMainPage = new BrowseMainPage(parent, q->actionCollection(), mGvCore);
 
         mThumbnailView = mBrowseMainPage->thumbnailView();
+        mThumbnailView->viewport()->installEventFilter(q);
         mThumbnailView->setSelectionModel(mContextManager->selectionModel());
         mUrlNavigator = mBrowseMainPage->urlNavigator();
 
@@ -283,15 +287,15 @@ struct MainWindow::Private
         mThumbnailView->setDocumentInfoProvider(mDocumentInfoProvider);
 
         mThumbnailViewHelper = new ThumbnailViewHelper(mDirModel, q->actionCollection());
-        connect(mContextManager, SIGNAL(currentDirUrlChanged(QUrl)),
-            mThumbnailViewHelper, SLOT(setCurrentDirUrl(QUrl)));
+        connect(mContextManager, &ContextManager::currentDirUrlChanged,
+            mThumbnailViewHelper, &ThumbnailViewHelper::setCurrentDirUrl);
         mThumbnailView->setThumbnailViewHelper(mThumbnailViewHelper);
 
         mThumbnailBarSelectionModel = new KLinkItemSelectionModel(mThumbnailBarModel, mContextManager->selectionModel(), q);
 
         // Connect thumbnail view
-        connect(mThumbnailView, SIGNAL(indexActivated(QModelIndex)),
-                q, SLOT(slotThumbnailViewIndexActivated(QModelIndex)));
+        connect(mThumbnailView, &ThumbnailView::indexActivated,
+                q, &MainWindow::slotThumbnailViewIndexActivated);
 
         // Connect delegate
         QAbstractItemDelegate* delegate = mThumbnailView->itemDelegate();
@@ -307,21 +311,21 @@ struct MainWindow::Private
                 mGvCore, SLOT(setRating(QUrl,int)));
 
         // Connect url navigator
-        connect(mUrlNavigator, SIGNAL(urlChanged(QUrl)),
-                q, SLOT(openDirUrl(QUrl)));
+        connect(mUrlNavigator, &KUrlNavigator::urlChanged,
+                q, &MainWindow::openDirUrl);
     }
 
     void setupViewMainPage(QWidget* parent)
     {
         mViewMainPage = new ViewMainPage(parent, mSlideShow, q->actionCollection(), mGvCore);
-        connect(mViewMainPage, SIGNAL(captionUpdateRequested(QString)),
-                q, SLOT(slotUpdateCaption(QString)));
-        connect(mViewMainPage, SIGNAL(completed()),
-                q, SLOT(slotPartCompleted()));
-        connect(mViewMainPage, SIGNAL(previousImageRequested()),
-                q, SLOT(goToPrevious()));
-        connect(mViewMainPage, SIGNAL(nextImageRequested()),
-                q, SLOT(goToNext()));
+        connect(mViewMainPage, &ViewMainPage::captionUpdateRequested,
+                q, &MainWindow::slotUpdateCaption);
+        connect(mViewMainPage, &ViewMainPage::completed,
+                q, &MainWindow::slotPartCompleted);
+        connect(mViewMainPage, &ViewMainPage::previousImageRequested,
+                q, &MainWindow::goToPrevious);
+        connect(mViewMainPage, &ViewMainPage::nextImageRequested,
+                q, &MainWindow::goToNext);
         connect(mViewMainPage, &ViewMainPage::openUrlRequested,
                 q, &MainWindow::openUrl);
         connect(mViewMainPage, &ViewMainPage::openDirUrlRequested,
@@ -345,8 +349,8 @@ struct MainWindow::Private
     void setupStartMainPage(QWidget* parent)
     {
         mStartMainPage = new StartMainPage(parent, mGvCore);
-        connect(mStartMainPage, SIGNAL(urlSelected(QUrl)),
-                q, SLOT(slotStartMainPageUrlSelected(QUrl)));
+        connect(mStartMainPage, &StartMainPage::urlSelected,
+                q, &MainWindow::slotStartMainPageUrlSelected);
         connect(mStartMainPage, &StartMainPage::recentFileRemoved, [this](const QUrl& url) {
             mFileOpenRecentAction->removeUrl(url);
         });
@@ -373,7 +377,7 @@ struct MainWindow::Private
         mFileOpenRecentAction = KStandardAction::openRecent(q, SLOT(openUrl(QUrl)), q);
         connect(mFileOpenRecentAction, &KRecentFilesAction::recentListCleared,
                 mGvCore, &GvCore::clearRecentFilesAndFolders);
-        QAction* clearAction = mFileOpenRecentAction->menu()->findChild<QAction*>("clear_action");
+        QAction * clearAction = mFileOpenRecentAction->menu()->findChild<QAction*>("clear_action");
         if (clearAction) {
             clearAction->setText(i18nc("@action Open Recent menu", "Forget All Files && Folders"));
         }
@@ -385,6 +389,11 @@ struct MainWindow::Private
         action->setText(i18nc("@action reload the currently viewed image", "Reload"));
         action->setIcon(QIcon::fromTheme("view-refresh"));
         actionCollection->setDefaultShortcuts(action, KStandardShortcut::reload());
+        
+        QAction * replaceLocationAction = actionCollection->addAction(QStringLiteral("replace_location"));
+        replaceLocationAction->setText(i18nc("@action:inmenu Navigation Bar", "Replace Location"));
+        actionCollection->setDefaultShortcut(replaceLocationAction, Qt::CTRL + Qt::Key_L);
+        connect(replaceLocationAction, &QAction::triggered, q, &MainWindow::replaceLocation);
 
         mBrowseAction = view->addAction("browse");
         mBrowseAction->setText(i18nc("@action:intoolbar Switch to file list", "Browse"));
@@ -392,8 +401,8 @@ struct MainWindow::Private
         mBrowseAction->setCheckable(true);
         mBrowseAction->setIcon(QIcon::fromTheme("view-list-icons"));
         actionCollection->setDefaultShortcut(mBrowseAction, Qt::Key_Escape);
-        connect(mViewMainPage, SIGNAL(goToBrowseModeRequested()),
-            mBrowseAction, SLOT(trigger()));
+        connect(mViewMainPage, &ViewMainPage::goToBrowseModeRequested,
+            mBrowseAction, &QAction::trigger);
 
         mViewAction = view->addAction("view");
         mViewAction->setText(i18nc("@action:intoolbar Switch to image view", "View"));
@@ -405,16 +414,16 @@ struct MainWindow::Private
         mViewModeActionGroup->addAction(mBrowseAction);
         mViewModeActionGroup->addAction(mViewAction);
 
-        connect(mViewModeActionGroup, SIGNAL(triggered(QAction*)),
-                q, SLOT(setActiveViewModeAction(QAction*)));
+        connect(mViewModeActionGroup, &QActionGroup::triggered,
+                q, &MainWindow::setActiveViewModeAction);
 
         mFullScreenAction = KStandardAction::fullScreen(q, &MainWindow::toggleFullScreen, q, actionCollection);
         QList<QKeySequence> shortcuts = mFullScreenAction->shortcuts();
         shortcuts.append(QKeySequence(Qt::Key_F11));
         actionCollection->setDefaultShortcuts(mFullScreenAction, shortcuts);
 
-        connect(mViewMainPage, SIGNAL(toggleFullScreenRequested()),
-                mFullScreenAction, SLOT(trigger()));
+        connect(mViewMainPage, &ViewMainPage::toggleFullScreenRequested,
+                mFullScreenAction, &QAction::trigger);
 
         QAction * leaveFullScreenAction = view->addAction("leave_fullscreen", q, SLOT(leaveFullScreen()));
         leaveFullScreenAction->setIcon(QIcon::fromTheme("view-restore"));
@@ -467,15 +476,15 @@ struct MainWindow::Private
         mToggleSideBarAction->setIcon(QIcon::fromTheme("view-sidetree"));
         actionCollection->setDefaultShortcut(mToggleSideBarAction, Qt::Key_F4);
         mToggleSideBarAction->setText(i18nc("@action", "Sidebar"));
-        connect(mBrowseMainPage->toggleSideBarButton(), SIGNAL(clicked()),
-                mToggleSideBarAction, SLOT(trigger()));
-        connect(mViewMainPage->toggleSideBarButton(), SIGNAL(clicked()),
-                mToggleSideBarAction, SLOT(trigger()));
+        connect(mBrowseMainPage->toggleSideBarButton(), &QAbstractButton::clicked,
+                mToggleSideBarAction, &QAction::trigger);
+        connect(mViewMainPage->toggleSideBarButton(), &QAbstractButton::clicked,
+                mToggleSideBarAction, &QAction::trigger);
 
         mToggleSlideShowAction = view->addAction("toggle_slideshow", q, SLOT(toggleSlideShow()));
         q->updateSlideShowAction();
-        connect(mSlideShow, SIGNAL(stateChanged(bool)),
-                q, SLOT(updateSlideShowAction()));
+        connect(mSlideShow, &SlideShow::stateChanged,
+                q, &MainWindow::updateSlideShowAction);
 
         q->setStandardToolBarMenuEnabled(true);
 
@@ -551,7 +560,7 @@ struct MainWindow::Private
         page = new SideBarPage(i18n("Folders"));
         page->setObjectName(QLatin1String("folders"));
         page->addWidget(folderViewItem->widget());
-        page->layout()->setMargin(0);
+        page->layout()->setContentsMargins(0, 0, 0, 0);
         mSideBar->addPage(page);
 
         page = new SideBarPage(i18n("Information"));
@@ -581,13 +590,13 @@ struct MainWindow::Private
             | MimeTypeUtils::KIND_SVG_IMAGE
             | MimeTypeUtils::KIND_VIDEO);
 
-        connect(mDirModel, SIGNAL(rowsInserted(QModelIndex,int,int)),
-                q, SLOT(slotDirModelNewItems()));
+        connect(mDirModel, &QAbstractItemModel::rowsInserted,
+                q, &MainWindow::slotDirModelNewItems);
 
-        connect(mDirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
-                q, SLOT(updatePreviousNextActions()));
-        connect(mDirModel, SIGNAL(modelReset()),
-                q, SLOT(updatePreviousNextActions()));
+        connect(mDirModel, &QAbstractItemModel::rowsRemoved,
+                q, &MainWindow::updatePreviousNextActions);
+        connect(mDirModel, &QAbstractItemModel::modelReset,
+                q, &MainWindow::updatePreviousNextActions);
 
         connect(mDirModel->dirLister(), SIGNAL(completed()),
                 q, SLOT(slotDirListerCompleted()));
@@ -851,8 +860,8 @@ MainWindow::MainWindow()
     createGUI();
     loadConfig();
 
-    connect(DocumentFactory::instance(), SIGNAL(modifiedDocumentListChanged()),
-            SLOT(slotModifiedDocumentListChanged()));
+    connect(DocumentFactory::instance(), &DocumentFactory::modifiedDocumentListChanged,
+            this, &MainWindow::slotModifiedDocumentListChanged);
 
 #ifdef HAVE_QTDBUS
     d->mMpris2Service = new Mpris2Service(d->mSlideShow, d->mContextManager,
@@ -959,7 +968,7 @@ void MainWindow::setActiveViewModeAction(QAction* action)
         d->mViewStackedWidget->setCurrentWidget(d->mViewMainPage);
         openSelectedDocuments();
         d->mPreloadDirectionIsForward = true;
-        QTimer::singleShot(VIEW_PRELOAD_DELAY, this, SLOT(preloadNextUrl()));
+        QTimer::singleShot(VIEW_PRELOAD_DELAY, this, &MainWindow::preloadNextUrl);
     } else {
         d->mCurrentMainPageId = BrowseMainPageId;
         // Switching to browse mode
@@ -1215,7 +1224,7 @@ void MainWindow::slotSelectionChanged()
 
     // Start preloading
     int preloadDelay = d->mCurrentMainPageId == ViewMainPageId ? VIEW_PRELOAD_DELAY : BROWSE_PRELOAD_DELAY;
-    QTimer::singleShot(preloadDelay, this, SLOT(preloadNextUrl()));
+    QTimer::singleShot(preloadDelay, this, &MainWindow::preloadNextUrl);
 }
 
 void MainWindow::slotCurrentDirUrlChanged(const QUrl &url)
@@ -1239,7 +1248,7 @@ void MainWindow::slotDirListerCompleted()
 {
     if (d->mStartSlideShowWhenDirListerCompleted) {
         d->mStartSlideShowWhenDirListerCompleted = false;
-        QTimer::singleShot(0, d->mToggleSlideShowAction, SLOT(trigger()));
+        QTimer::singleShot(0, d->mToggleSlideShowAction, &QAction::trigger);
     }
     if (d->mContextManager->selectionModel()->hasSelection()) {
         updatePreviousNextActions();
@@ -1513,7 +1522,7 @@ void MainWindow::showConfigDialog()
     saveConfig();
     
     DialogGuard<ConfigDialog> dialog(this);
-    connect(dialog.data(), SIGNAL(settingsChanged(QString)), SLOT(loadConfig()));
+    connect(dialog.data(), &KConfigDialog::settingsChanged, this, &MainWindow::loadConfig);
     dialog->exec();
 }
 
@@ -1662,9 +1671,49 @@ bool MainWindow::eventFilter(QObject *obj, QEvent *event)
         return true;
     }
 #endif
+    if (obj == d->mThumbnailView->viewport()) {
+        switch(event->type()) {
+        case QEvent::MouseButtonPress:
+        case QEvent::MouseButtonDblClick:
+            mouseButtonNavigate(static_cast<QMouseEvent*>(event));
+            break;
+        default: ;
+        }
+    }
     return false;
 }
 
+void MainWindow::mousePressEvent(QMouseEvent *event)
+{
+    mouseButtonNavigate(event);
+    KXmlGuiWindow::mousePressEvent(event);
+}
+
+void MainWindow::mouseDoubleClickEvent(QMouseEvent *event)
+{
+    mouseButtonNavigate(event);
+    KXmlGuiWindow::mouseDoubleClickEvent(event);
+}
+
+void MainWindow::mouseButtonNavigate(QMouseEvent *event)
+{
+    switch(event->button()) {
+    case Qt::ForwardButton:
+        if (d->mGoToNextAction->isEnabled()) {
+            d->mGoToNextAction->trigger();
+            return;
+        }
+        break;
+    case Qt::BackButton:
+        if (d->mGoToPreviousAction->isEnabled()) {
+            d->mGoToPreviousAction->trigger();
+            return;
+        }
+        break;
+    default: ;
+    }
+}
+
 void MainWindow::setDistractionFreeMode(bool value)
 {
     d->mFullScreenContent->setDistractionFreeMode(value);
@@ -1721,4 +1770,21 @@ void MainWindow::showLastDocumentReached()
     d->mViewMainPage->showMessageWidget(dlg, Qt::AlignCenter);
 }
 
+void MainWindow::replaceLocation()
+{
+    QLineEdit* lineEdit = d->mUrlNavigator->editor()->lineEdit();
+
+    // If the text field currently has focus and everything is selected,
+    // pressing the keyboard shortcut returns the whole thing to breadcrumb mode
+    if (d->mUrlNavigator->isUrlEditable()
+        && lineEdit->hasFocus()
+        && lineEdit->selectedText() == lineEdit->text() ) {
+        d->mUrlNavigator->setUrlEditable(false);
+    } else {
+        d->mUrlNavigator->setUrlEditable(true);
+        d->mUrlNavigator->setFocus();
+        lineEdit->selectAll();
+    }
+}
+
 } // namespace
