@@ -105,7 +105,7 @@ void SaveAllHelper::slotResult(KJob* _job)
         QUrl url = job->document()->url();
         QString name = url.fileName().isEmpty() ? url.toDisplayString() : url.fileName();
         d->mErrorList << xi18nc("@info %1 is the name of the document which failed to save, %2 is the reason for the failure",
-                                "<filename>%1</filename>: %2", name, job->errorString());
+                                "<filename>%1</filename>: %2", name, kxi18n(qPrintable(job->errorString())));
     }
     d->mJobSet.remove(job);
     d->mProgressDialog->setValue(d->mProgressDialog->value() + 1);
