@@ -596,10 +596,10 @@ PreviewItemDelegate::PreviewItemDelegate(ThumbnailView* view)
     d->mContextBarActions = SelectionAction | FullScreenAction | RotateAction;
     d->mTextElideMode = Qt::ElideRight;
 
-    connect(view, SIGNAL(rowsRemovedSignal(QModelIndex,int,int)),
-            SLOT(slotRowsChanged()));
-    connect(view, SIGNAL(rowsInsertedSignal(QModelIndex,int,int)),
-            SLOT(slotRowsChanged()));
+    connect(view, &ThumbnailView::rowsRemovedSignal,
+            this, &PreviewItemDelegate::slotRowsChanged);
+    connect(view, &ThumbnailView::rowsInsertedSignal,
+            this, &PreviewItemDelegate::slotRowsChanged);
     connect(view, &ThumbnailView::selectionChangedSignal, [this]() {
         d->updateToggleSelectionButton();
     });
@@ -610,8 +610,8 @@ PreviewItemDelegate::PreviewItemDelegate(ThumbnailView* view)
     d->mRatingPainter.setMaxRating(10);
 #endif
 
-    connect(view, SIGNAL(thumbnailSizeChanged(QSize)),
-            SLOT(setThumbnailSize(QSize)));
+    connect(view, &ThumbnailView::thumbnailSizeChanged,
+            this, &PreviewItemDelegate::setThumbnailSize);
 
     // Button frame
     d->mContextBar = new QWidget(d->mView->viewport());
@@ -724,16 +724,16 @@ void PreviewItemDelegate::paint(QPainter * painter, const QStyleOptionViewItem &
         bgColor = option.palette.color(cg, QPalette::Highlight);
 
         if (hasFocus) {
-            borderColor = bgColor.dark(FOCUS_BORDER_DARKNESS);
+            borderColor = bgColor.darker(FOCUS_BORDER_DARKNESS);
         } else {
-            borderColor = bgColor.dark(SELECTION_BORDER_DARKNESS);
+            borderColor = bgColor.darker(SELECTION_BORDER_DARKNESS);
         }
     } else {
         bgColor = viewport->palette().color(viewport->backgroundRole());
         if (hasFocus) {
             borderColor = fgColor;
         } else {
-            borderColor = bgColor.light(200);
+            borderColor = bgColor.lighter(200);
         }
     }
 
